<?php
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

class CashBill extends PaymentModule
{
    protected $_html = '';
    protected $_postErrors = array();

    public $details;
    public $owner;
    public $address;
    public $extra_mail_vars;

    const CB_MAIN_URL = "https://pay.cashbill.pl/ws/rest";
    const CB_TEST_URL = "https://pay.cashbill.pl/testws/rest";

    public function __construct()
    {
        $this->name = 'cashbill';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.2';
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
        $this->author = 'CashBill S.A.';
        $this->controllers = array('validation','notify');
        $this->is_eu_compatible = 1;

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('Płatności CashBill');
        $this->description = $this->l('Błyskawiczne płatności dla Twojego sklepu internetowego!');

        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('Brak walut ustawionych dla tego modułu!');
        }
    }

    public function install()
    {
        if (!parent::install() || !$this->registerHook('paymentOptions') || !$this->registerHook('paymentReturn') || !Configuration::updateValue('CB_ID', '') ||
        !Configuration::updateValue('CB_SECRET_PHRASE', '') ||
        !Configuration::updateValue('CB_TEST_MODE', true)
        ) {
            return false;
        }

        if (Validate::isInt(Configuration::get('PAYMENT_CASHBILL_NEW_STATUS')) xor ( Validate::isLoadedObject($order_state_new = new OrderState(Configuration::get('PAYMENT_CASHBILL_NEW_STATUS'))))) {
            $order_state_new = new OrderState();
            $order_state_new->name[Language::getIdByIso("pl")] = "Oczekuje potwierdzenia płatności";
            $order_state_new->name[Language::getIdByIso("en")] = "Awaiting payment confirmation";
            $order_state_new->send_email = false;
            $order_state_new->invoice = false;
            $order_state_new->unremovable = false;
            $order_state_new->color = "lightblue";
            if (!$order_state_new->add()) {
                return false;
            }
            if (!Configuration::updateValue('PAYMENT_CASHBILL_NEW_STATUS', $order_state_new->id)) {
                return false;
            }
        }

        return true;
    }

    public function hookPaymentReturn()
    {
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $payment_options = [
            $this->getExternalPaymentOption()
        ];

        return $payment_options;
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function getExternalPaymentOption()
    {
        $externalOption = new PaymentOption();
        $externalOption->setCallToActionText($this->l('Natychmiastowe płatności internetowe CashBill'))
                       ->setAction($this->context->link->getModuleLink($this->name, 'validation', array(), true))
                       ->setAdditionalInformation($this->context->smarty->fetch('module:cashbill/views/templates/front/payment_infos.tpl'))
                       ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/logo.png'));

        return $externalOption;
    }
    public function isPropValid($prop)
    {
        return $prop && !empty($prop);
    }

    public function getContent()
    {
        $output = null;
    
        if (Tools::isSubmit('submit'.$this->name)) {
            $cb_id = strval(Tools::getValue('CB_ID'));
            $cb_secret = strval(Tools::getValue('CB_SECRET_PHRASE'));
            $cb_mode = strval(Tools::getValue('CB_TEST_MODE'));
            if ($this->isPropValid($cb_id) && $this->isPropValid($cb_secret)) {
                Configuration::updateValue('CB_ID', $cb_id);
                Configuration::updateValue('CB_SECRET_PHRASE', $cb_secret);
                Configuration::updateValue('CB_TEST_MODE', $cb_mode ? true : false);
                $output .= $this->displayConfirmation($this->l('Ustawienia zaktualizowane.'));
            } else {
                $output .= $this->displayError($this->l('Wypełnij poprawnie wszystkie pola.'));
            }
        }
        return $output.$this->displayForm();
    }

    public function displayForm()
    {
        // Get default language
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
    
        // Init Fields form array
        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Ustawienia'),
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('Identyfikator Punktu Płatności'),
                    'name' => 'CB_ID',
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Klucz Punktu Płatności'),
                    'name' => 'CB_SECRET_PHRASE',
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Tryb Testowy Włączony'),
                    'name' => 'CB_TEST_MODE',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Yes')
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('No')
                        )
                    ),
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            )
        );
    
        $helper = new HelperForm();
    
        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
    
        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
    
        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = array(
            'save' =>
            array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );
    
        // Load current value
        $helper->fields_value['CB_ID'] = Configuration::get('CB_ID');
        $helper->fields_value['CB_SECRET_PHRASE'] = Configuration::get('CB_SECRET_PHRASE');
        $helper->fields_value['CB_TEST_MODE'] = Configuration::get('CB_TEST_MODE');
        
        return $helper->generateForm($fields_form);
    }

    public static function getPayment($id)
    {
        $mainUrl = Configuration::get('CB_TEST_MODE') ? self::CB_TEST_URL : self::CB_MAIN_URL;
        $shopId = Configuration::get('CB_ID');
        $secretPhrase = Configuration::get('CB_SECRET_PHRASE');

        $sign = sha1($id . $secretPhrase);

        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_URL => "{$mainUrl}/payment/{$shopId}/{$id}?sign={$sign}",
        ));

        $out = curl_exec($ch);

        if (curl_getinfo($ch, CURLINFO_HTTP_CODE) != 200) {
            curl_close($ch);
            throw new Exception($out);
        }
        curl_close($ch);

        return json_decode($out);
    }
}
